#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

static GtkWidget *dialogbox;

void debug(char *format, ...)
{
   va_list ap;
   
#ifdef DEBUG
   va_start(ap,format);
   vfprintf(stderr,format, ap);
   va_end(ap);
#endif
}

void ldebug(int level, char *format, ...)
{
   va_list ap;
   
   if (DEBUGLEVEL < level)
     return;
   va_start(ap,format);
   vfprintf(stderr,format, ap);
   va_end(ap);
}

void defaultAction(GtkButton *button, gpointer user_data)
{
   GtkWidget *dialogbox = 
     GTK_WIDGET(lookup_widget (GTK_WIDGET(button), "dialogbox"));
   if(dialogbox == NULL)
     return;
   gtk_widget_hide(dialogbox);
   gtk_widget_destroy(dialogbox);
}

void createMessageBox(char *text)
{
   GtkWidget *button;
   
   dialogbox = create_dialogbox();
   
   /* Turn on the buttons we want here : */
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogclose")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogcancel")));
    gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogok")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogyes")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogno")));
   
   gtk_widget_show(dialogbox);
   
   if(text != NULL)
     gtk_label_set_text
     (GTK_LABEL(lookup_widget(dialogbox, "dialogtext")), text);
   
   button =lookup_widget (GTK_WIDGET (dialogbox), "dialogok");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (defaultAction), NULL);
}

void createConfirmBox(char *text, gpointer yesAction,
		      gpointer noAction, gpointer user_data)
{
   GtkWidget *button;
   
   if(yesAction == NULL)
     yesAction = defaultAction;
   if(noAction == NULL)
     noAction = defaultAction;
   
   dialogbox = create_dialogbox();
   
   /* Turn on the buttons we want here : */
// gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogclose")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogcancel")));
// gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogok")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogyes")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogno")));
   
   gtk_widget_show(dialogbox);
   
   if(text != NULL)
     gtk_label_set_text
     (GTK_LABEL(lookup_widget(dialogbox, "dialogtext")), text);
   
   button =lookup_widget (GTK_WIDGET (dialogbox), "dialogcancel");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (defaultAction), NULL);
   
   button =lookup_widget (GTK_WIDGET (dialogbox), "dialogyes");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (yesAction), user_data);

   button =lookup_widget (GTK_WIDGET (dialogbox), "dialogno");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (noAction), user_data);
}

/* This should be a notice dialogbox */
void showMessage(char *s,...)
{
   va_list ap;
   char message[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(message,sizeof(message) - 1, s, ap);
   ldebug(2,"Message: %s\n",message);
   
   createMessageBox(message);
}

void showSqlError(GtkText *logw, SQL *sock, char *s,...)
{
   va_list ap;
   char error[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(error,sizeof(error) - 1, s, ap);
   
   writeLog(logw,"Error: %s\n",error);
   writeLog(logw,"Error: %s\n",sql_error(sock));
}

/* This should go to the logbrowser */
void writeLog(GtkText *logw, char *s,...)
{
   va_list ap;
   GtkAdjustment *logvbar;
   char logline[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(logline,sizeof(logline) - 1, s, ap);
   ldebug(2,"%s\n",logline);
   
   gtk_text_freeze (logw);
   gtk_text_insert (logw,NULL, NULL, NULL, logline, -1);
 
   /* Clear toplines regularly to avoid heavy memory use */
   if(gtk_text_get_length (logw) > MAXLOGSIZE)
   {
      ldebug(2,"Logsize larger than %d: removing top lines ...\n",MAXLOGSIZE);
      gtk_text_set_point (logw,0); /* goto begin of text */
      gtk_text_forward_delete (logw, MAXLOGSIZE/4); /* remove 1/4 */
   }

   gtk_text_thaw (logw);

   /* Always scroll to end of logwindow : */
   /* Note: this must be AFTER gtk_text_thaw or else we use the old end ! */
   logvbar =  gtk_scrolled_window_get_vadjustment
     (GTK_SCROLLED_WINDOW
      (lookup_widget(GTK_WIDGET(logw), "logscroller")));
   gtk_adjustment_set_value (logvbar, logvbar->upper);
}

void saveLogAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   Params *p = (Params *) user_data;
   
   ldebug(2,"saveLogAction entered\n");
   
   output = fopen(p->selectedname,"w");
   if(output == NULL)
   {
      showMessage("Cannot save log:\n open file %s failed !\n",
		  p->selectedname);
      return;
   }
   
   fprintf(output,gtk_editable_get_chars(GTK_EDITABLE(p->logwindow),0,-1));
   
   fclose(output);
}

