#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

int getIntValue(GtkWidget *widget, char *widgetname)
{
   GtkWidget *target =lookup_widget (GTK_WIDGET (widget), widgetname);
   return gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (target));
}

char *getStringValue(GtkWidget *widget, char *widgetname)
{
   GtkWidget *target =lookup_widget (GTK_WIDGET (widget), widgetname);
   return  gtk_entry_get_text (GTK_ENTRY (target));
}

void setStringValue(GtkWidget *widget, char *widgetname,char *string)
{
   GtkWidget *target =lookup_widget (GTK_WIDGET (widget), widgetname);
   gtk_entry_set_text (GTK_ENTRY (target), string);
}

int executeQuery(GtkText *logw, SQL *sock,char *query)
{   
   writeLog(logw,"Executing: %s\n",query);
   if (sql_query(sock,query) < 0)
   {
      showSqlError(logw,sock,"Query failed");
      return -1;
   }
   return 0;
}

void closeTopWindow(GtkWidget *child)
{
   GtkWidget *parent;
   
   while(TRUE)
   {
      if (GTK_IS_MENU (child))
	parent = gtk_menu_get_attach_widget (GTK_MENU (child));
      else
	parent = child->parent;
      if (parent == NULL)
	break;
      child = parent;
   }
   
   gtk_widget_hide(child);
   gtk_widget_destroy(child);
}

int getNumRecords(Params *p)
{
   SQL_RES *res;
   char query[MAXQUERY];
   
   ldebug(2,"getNumRecords called\n");
   if(p->countflag == NOCOUNT)
     return 0;
   
   snprintf(query,sizeof(query) - 1,SELECT,p->table);
   
   if (executeQuery(p->logwindow, p->dbsock, query) < 0)
     return 0;
   
   res = sql_store_result(p->dbsock);
   if(res == NULL)
   {
      showSqlError(p->logwindow,p->dbsock,
		   "getNumRecords: Cannot store result");
      return 0;
   }

   p->numrecs = sql_num_rows(res);
   
   sql_free_result(res);
   return p->numrecs;
}

void setGUICounters(Params *p, int nrFields, int nrRecords)
{
   char string[10]; /* should be enough ;-) */
   
   ldebug(2,"setGUICounters called\n");
   snprintf(string,sizeof(string) - 1,"%d",nrFields);
   setStringValue(p->mainwindow, "fieldcount", string);
   
   snprintf(string,sizeof(string) - 1,"%d",nrRecords);
   setStringValue(p->mainwindow, "recordcount", string);  
}

int getSortKey(Params *p)
{
   int y;
   
   ldebug(2,"getSortKey called\n");
   /* Find unique primary key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_PRI_KEY(p->fields[y].flags)&&IS_UNIQUE(p->fields[y].flags))
	return y;
   }
   
   /* Else, find first unique key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_UNIQUE(p->fields[y].flags))
	return y;
   }
   
   /* Else, find primary key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_PRI_KEY(p->fields[y].flags))
	return y;
   }
   
   writeLog(p->logwindow,"Notice: No unique or primary key in %s,"
	    "using first field.\n", p->table);
   return 0;
}

void destroyBrowser(Params *p, int idx)
{
   ldebug(2,"destroyBrowser entered\n");
   if(p->tabinuse[idx] == FALSE)
     return;
   
   if(! GTK_IS_WIDGET(p->clists[idx]))
   {
      p->clists[idx] = NULL;
      return;
   }
   
   gtk_widget_hide(p->clists[idx]);
   gtk_widget_destroy(p->clists[idx]);

   p->tabinuse[idx] = FALSE;
}

void destroyAllBrowsers(Params *p)
{
   int x;
   
   for(x=0;x<p->nrtabs;x++)
     destroyBrowser(p, x);
}

GtkStyle *createFontStyle(GtkStyle *source, gchar *fontname)
{
   GtkStyle *style= gtk_style_copy(source);

   gdk_font_unref (style->font);
   style->font = gdk_font_load(fontname);

   return style;
}

GtkStyle *createColorStyle(GtkStyle *source, double rgb[3])
{
   GtkStyle *style= gtk_style_copy(source);
   
   GdkColor *c = (GdkColor*) g_malloc(sizeof(GdkColor));
   c->red   = (int)(rgb[0]*65535);
   c->green = (int)(rgb[1]*65535);
   c->blue  = (int)(rgb[2]*65535);
   
   style->fg[GTK_STATE_NORMAL] = *c;
   
   return style;
}

GtkStyle *createColorFontStyle(GtkStyle *source, double rgb[3], gchar *fontname)
{
   GtkStyle *style= createColorStyle(source, rgb);
   
   gdk_font_unref (style->font);
   style->font = gdk_font_load(fontname);

   return style;
}

