#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

void setSortKey(Params *p, int row)
{
   p->sortkey = row;
   showFields(p);
}

/* If we come here, then the user has selected a row in the list. */
void field_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
   Params *p = (Params *)data;

   ldebug(2,"field_selected entered\n");

   /* This only catches <SPACE> or a doubleclick on the selected record !! : */
   if (event == NULL || event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"field_selected: doubleclick detected\n");
      
      if(p->table == NULL)
      {
	 showMessage("Currently no table selected !\n");
	 return;
      }
      setSortKey(p, row);
      writeLog(p->logwindow, "Sort field now set to field %d (%s)\n",
	       row,p->fields[row].name);
      return;
   }
   else
      ldebug(3,"You selected field %d: no effect (use doubleclick !)\n",row);
}

gchar *getFieldTypeString(Params *p, int index)
{
   switch(p->fields[index].type)
   {
    case FIELD_TYPE_STRING:     return "String";    break;
    case FIELD_TYPE_VAR_STRING: return "VarString"; break;
    case FIELD_TYPE_FLOAT:      return "Float";     break;
    case FIELD_TYPE_DOUBLE:     return "Double";    break;
    case FIELD_TYPE_LONGLONG:
    case FIELD_TYPE_INT24:
    case FIELD_TYPE_DECIMAL:
    case FIELD_TYPE_TINY:
    case FIELD_TYPE_SHORT:
    case FIELD_TYPE_LONG:       return "Integer";   break;
    case FIELD_TYPE_DATE:       return "Date";      break;
    case FIELD_TYPE_TIME:       return "Time";      break;
    case FIELD_TYPE_TIMESTAMP:  return "TimeStamp"; break;
    case FIELD_TYPE_DATETIME:   return "Datetime";  break;
    case FIELD_TYPE_YEAR:       return "Year";      break;
    case FIELD_TYPE_TINY_BLOB:
    case FIELD_TYPE_MEDIUM_BLOB:
    case FIELD_TYPE_LONG_BLOB:
    case FIELD_TYPE_BLOB:       return "Blob:";     break;
    default:
      writeLog(p->logwindow,"Unknown fieldtype %d detected !\n",
	       p->fields[index].type);
      return "Unknown"; break;
   }
}

void destroyFieldBrowser(Params *p)
{
   ldebug(2,"destroyFieldBrowser entered\n");
   destroyBrowser(p, FIELDSTAB);
}

void createFieldBrowser(Params *p, gchar **titles, void *callback, int size)
{
   GtkWidget *fb;
   
   ldebug(2,"createFieldBrowser entered\n");
   
   fb = GTK_WIDGET(gtk_clist_new_with_titles(size, titles));

   gtk_signal_connect(GTK_OBJECT(fb), "select_row",
		      GTK_SIGNAL_FUNC(callback),
		      (gpointer) p);
   
   gtk_widget_ref (fb);
   gtk_object_set_data_full (GTK_OBJECT(p->mainwindow), "fieldsclist", fb,
			     (GtkDestroyNotify) gtk_widget_unref);
   
   gtk_widget_show (fb);
  
   gtk_container_add(GTK_CONTAINER(p->tabs[FIELDSTAB]), fb);
   
   p->clists[FIELDSTAB]   = fb;
   p->tabinuse[FIELDSTAB] = TRUE;
}

void setFieldColumnWidths(Params *p, int numcolumns, gchar **titles)
{
   int x;
   int fntwidth = FONTWIDTH; /* fixed fontsize for the time being */
   int cwidth;
   
   /* set column sizes according to the fieldlength : */
   for(x=0; x < numcolumns; x++)
   {
      cwidth = (strlen(titles[x]) + 5) * fntwidth;
      ldebug(3,"Column width for column %d : %d\n",x,cwidth);
      gtk_clist_set_column_width(GTK_CLIST(p->clists[FIELDSTAB]), x, cwidth);
   }
}

void showFields(Params *p)
{
   gchar length[10];
   int   x,numrows;
   int   numcolumns = 6;
   gchar *row[6];
   gchar *titles[6] =
   { "Fieldname", "Tablename", "Length", "Type", "Unique", "Not Null" };
   
   ldebug(2,"showFields entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show fields: select a table first !\n");
      return;
   }
   
   numrows = p->numflds; /* number of browserlines */
   
   destroyFieldBrowser(p);
   createFieldBrowser(p,titles,field_selected,numcolumns);
   setFieldColumnWidths(p,numcolumns,titles);
   
   gtk_clist_freeze(GTK_CLIST(p->clists[FIELDSTAB]));
   
   for(x=0; x < numrows; x++)
   {
      snprintf(length,sizeof(length) - 1,"%8d",p->fields[x].length);
      
      row[0] = p->fields[x].name;
      row[1] = p->table;
      row[2] = length;
      row[3] = getFieldTypeString(p, x);
      row[4] = IS_UNIQUE(p->fields[x].flags) ? "Y" : "N";
      row[5] = IS_NOT_NULL(p->fields[x].flags) ? "Y" : "N";
      ldebug(2,"field %d: type %d\n",x,p->fields[x].type);

      gtk_clist_append(GTK_CLIST(p->clists[FIELDSTAB]),row);
      /* change the sortkey color : */
      if(x == p->sortkey)
	gtk_clist_set_row_style(GTK_CLIST(p->clists[FIELDSTAB]),
				x, p->styles[STLRED]);
   }
   
   gtk_clist_thaw(GTK_CLIST(p->clists[FIELDSTAB]));
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), FIELDSTAB);
}

