#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

/* If we come here, then the user has selected a row in the list. */
void qrecord_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
/*   Params *p = (Params *)data; */
   
   ldebug(2,"qrecord_selected entered\n");
   
   /* This only catches <SPACE> or a doubleclick on the selected record !! : */
   if (event == NULL || event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"qrecord_selected: doubleclick detected\n");
      showMessage("Sorry: can't open free query records\n"
		  "in a separate window !\n");
   }
}

void destroyQueryBrowser(Params *p)
{
   ldebug(2,"destroyQueryBrowser entered\n");
   destroyBrowser(p, QUERYTAB);
   p->selectedrow = -1; /* TODO : use a separate variable for query !!!!! */
}

void createQueryBrowser(Params *p, gchar **titles, void *callback, int size)
{
   GtkWidget *qb;
   
   ldebug(2,"createQueryBrowser entered\n");
   
   qb = GTK_WIDGET(gtk_clist_new_with_titles(size, titles));

   gtk_signal_connect(GTK_OBJECT(qb), "select_row",
		      GTK_SIGNAL_FUNC(callback),
		      (gpointer) p);
   
   gtk_widget_ref (qb);
   gtk_object_set_data_full (GTK_OBJECT(p->mainwindow), "queryclist", qb,
			     (GtkDestroyNotify) gtk_widget_unref);
   
   gtk_widget_show (qb);
  
   gtk_container_add(GTK_CONTAINER(p->tabs[QUERYTAB]),qb);
   
   p->clists[QUERYTAB]   = qb;
   p->tabinuse[QUERYTAB] = TRUE;
}

/* TODO : combine with setRecordColumnWidths by using 2nd arg "clist" ? */
void setQueryColumnWidths(Params *p)
{
   int x;
   int fntwidth = FONTWIDTH; /* fixed fontsize for the time being */
   int cwidth;
   
   /* set column sizes according to the fieldlength : */
   for(x=0;x<p->numflds;x++)
   {
      cwidth = p->fields[x].length * fntwidth;
      ldebug(3,"Column width for column %d : %d\n",x,cwidth);
      gtk_clist_set_column_width(GTK_CLIST(p->clists[QUERYTAB]), x, cwidth);
   }
}

/* TODO : combine with showRecords ? (mind number of fields !) */
void showQuery(Params *p, char *query)
{
   int     x,numrows,numflds;
   SQL_ROW row;
   SQL_RES *res;
   gchar   **titles;
   
   ldebug(2,"showQuery entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show select query:\n select a table first !\n");
      return;
   }
   
   if(strncasecmp(query,"select ",7) != 0)
   {
      showMessage("Cannot execute query: Currently only\n"
		  "SELECT statements allowed. Sorry !\n");
      return;
   }
   
   if (executeQuery(p->logwindow, p->dbsock, query))
     return;
   
   res     = sql_store_result(p->dbsock);
   numflds = sql_num_fields(res); /* get number of fields of THIS query */
   
   /* Now fetch the fieldnames for this selection : */
   titles = (gchar **)g_malloc(sizeof(gchar *) * numflds);
   for(x=0;x<numflds;x++)
   {
      SQL_FIELD *field = sql_fetch_field(res);
      int tlen         = strlen(field->name) + 1;
      
      titles[x] = (char *)g_malloc(tlen);
      strncpy(titles[x], field->name, tlen);
   }
   
   destroyQueryBrowser(p); /* destroy the possible previous instance */
   createQueryBrowser(p,titles,qrecord_selected,numflds);
   setQueryColumnWidths(p);
   
   for(x=0;x<numflds;x++)
     g_free(titles[x]);
   g_free(titles);
   
   gtk_clist_freeze(GTK_CLIST(p->clists[QUERYTAB]));
   
   numrows = sql_num_rows(res);   /* get number of records for THIS query */
   for(x=0;x < numrows;x++)
   {
      if((row = sql_fetch_row(res)) == NULL)
      {
	 showSqlError(p->logwindow,p->dbsock,"Error reading record %d !",x);
	 return;
      }
      
      /* write a row to the browser : */
      gtk_clist_append(GTK_CLIST(p->clists[QUERYTAB]),row);
   }
   
   gtk_clist_thaw(GTK_CLIST(p->clists[QUERYTAB]));
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), QUERYTAB);
}

void loadQueryAction(GtkWidget *w, gpointer user_data)
{
   FILE *input;
   int length;
   char qline[MAXQUERY];
   Params *p = (Params *) user_data;
   
   ldebug(2,"loadQueryAction entered\n");
   
   input = fopen(p->selectedname,"r");
   if(input == NULL)
   {
      showMessage("Cannot load query:\n open file %s failed !\n",
		  p->selectedname);
      return;
   }
   
   gtk_text_set_point (p->querywindow,0); /* goto begin of text */
   length = gtk_text_get_length(p->querywindow);
   gtk_text_forward_delete (p->querywindow, length); /* clear all */
   
   if(fgets(qline,MAXQUERY-1,input) != NULL)
     gtk_text_insert (p->querywindow, NULL, NULL, NULL, qline, -1);
   
   fclose(input);
}

void saveQueryAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   Params *p = (Params *) user_data;
   
   ldebug(2,"saveQueryAction entered\n");
   
   output = fopen(p->selectedname,"w");
   if(output == NULL)
   {
      showMessage("Cannot save query:\n open file %s failed !\n",
		  p->selectedname);
      return;
   }
   
   fprintf(output,gtk_editable_get_chars(GTK_EDITABLE(p->querywindow),0,-1));
   
   fclose(output);
}

