#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

/* If we come here, then the user has selected a row in the list. */
void record_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
   Params *p = (Params *)data;

   ldebug(2,"record_selected entered\n");

   p->selectedrow = row; /* single click : just preserve selection */
   ldebug(3,"You selected row %d.\n ",row);
   
   /* This only catches <SPACE> or a doubleclick on the selected record !! : */
   if (event == NULL || event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"record_selected: doubleclick detected\n");
      
      if(p->table == NULL)
      {
	 showMessage("Currently no table selected !\n");
	 return;
      }
      
      ldebug(3,"You doubleclicked row %d. More specifically you clicked in "
	     "column %d\n\n", row, column);
      
      createModifyWindow(p);
   }
}

void destroyRecordBrowser(Params *p)
{
   ldebug(2,"destroyRecordBrowser entered\n");
   destroyBrowser(p, RECORDSTAB);
   p->selectedrow = -1;
}

void createRecordBrowser(Params *p, gchar **titles, void *callback, int size)
{
   GtkWidget *rb;
   
   ldebug(2,"createRecordBrowser entered\n");
   
   rb = GTK_WIDGET(gtk_clist_new_with_titles(size, titles));
   
   gtk_signal_connect(GTK_OBJECT(rb), "select_row",
		      GTK_SIGNAL_FUNC(callback), (gpointer) p);

/*  Change the sort key if the column title is clicked */
/*  gtk_signal_connect(GTK_OBJECT(rb),    "click-column",
                      GTK_SIGNAL_FUNC(setSortKeyAction),(gpointer) p);
 */
   gtk_widget_ref (rb);
   gtk_object_set_data_full (GTK_OBJECT(p->mainwindow), "recordsclist", rb,
			     (GtkDestroyNotify) gtk_widget_unref);
   
   gtk_widget_show (rb);
  
   gtk_container_add(GTK_CONTAINER(p->tabs[RECORDSTAB]), rb);
   
   p->clists[RECORDSTAB] = rb;
   p->tabinuse[RECORDSTAB] = TRUE;
}

void setRecordColumnWidths(Params *p)
{
   int x;
   int fntwidth = FONTWIDTH; /* fixed fontsize for the time being */
   int cwidth;
   
   /* set column sizes according to the fieldlength : */
   for(x=0;x<p->numflds;x++)
   {
      cwidth = p->fields[x].length * fntwidth;
      ldebug(3,"Column width for column %d : %d\n",x,cwidth);
      gtk_clist_set_column_width(GTK_CLIST(p->clists[RECORDSTAB]), x, cwidth);
   }
}

void showRecords(Params *p)
{
   int     x,numrows;
   SQL_ROW row;
   SQL_RES *res;
   gchar   **titles;
   char    query[MAXQUERY];
   
   ldebug(2,"showRecords entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show records:\n select a table first !\n");
      return;
   }
   
   snprintf(query,sizeof(query) - 1,SELECT ORDER,
	    p->table,p->fields[p->sortkey].name);
   
   if (executeQuery(p->logwindow, p->dbsock, query))
     return;
   
   res     = sql_store_result(p->dbsock);
   numrows = sql_num_rows(res);   /* get number of records in selection */

   titles = (gchar **)g_malloc(sizeof(gchar *) * p->numflds);
   for(x=0;x<p->numflds;x++)
     titles[x] = p->fields[x].name;

   destroyRecordBrowser(p); /* destroy the possible previous instance */
   createRecordBrowser(p,titles,record_selected,p->numflds);
   setRecordColumnWidths(p);
   g_free(titles);
   
   gtk_clist_freeze(GTK_CLIST(p->clists[RECORDSTAB]));
   
   for(x=0;x < numrows;x++)
   {
      if((row = sql_fetch_row(res)) == NULL)
      {
	 showSqlError(p->logwindow,p->dbsock,"Error reading record %d !",x);
	 return;
      }
      
      /* write a row to the browser : */
      gtk_clist_append(GTK_CLIST(p->clists[RECORDSTAB]),row);
   }
   
   gtk_clist_thaw(GTK_CLIST(p->clists[RECORDSTAB]));
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), RECORDSTAB);
}

void saveRecordsAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   Params *p = (Params *) user_data;
   int x,y;
   
   ldebug(2,"saveRecordsAction entered\n");
   
   if(p->table == NULL || p->tabinuse[RECORDSTAB] == FALSE)
   {
      showMessage("Cannot save records:\n no records in browser !\n");
      return;
   }
   
   output = fopen(p->selectedname,"w");
   if(output == NULL)
   {
      showMessage("Cannot save records:\n open file %s failed !\n",
		  p->selectedname);
      return;
   }
   
   for(y=0;y<p->numrecs; y++)
   {
      for(x=0;x<p->numflds;x++)
      {  
	 gchar *content;
	 
	 gtk_clist_get_text(GTK_CLIST(p->clists[RECORDSTAB]), y, x, &content);
	 fprintf(output,"%s\t",content);
	 ldebug(3,"%s\t",content);
      }
      fprintf(output,"\n");
      ldebug(3,"\n");
   }
   
   fclose(output);
}

