#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

/* First the callbacks for MainWindow : */

/* Menus in main window : */

void on_file_loadquery_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   createFileSelector((Params *)user_data, "query.out",
		      loadQueryAction, NULL);
}

void on_file_savequery_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   createFileSelector((Params *)user_data, "query.out",
		      saveQueryAction, NULL);
}

void on_file_saverecords_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   createFileSelector((Params *)user_data, "records.out",
		      saveRecordsAction, NULL);
}

void on_file_savelog_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   createFileSelector((Params *)user_data, "gequel.log",
		      saveLogAction, NULL);
}

void on_file_print_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   ;
}

void on_file_exit_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   ldebug(2,"on_exit_activate called\n");
   gtk_main_quit();
}

void on_edit_insert_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   createInsertWindow((Params *)user_data);
}

void on_edit_delete_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   Params *p = (Params *)user_data;
   
   if(p->tabinuse[RECORDSTAB] == FALSE || p->selectedrow < 0)
   {
      showMessage("Please open browser and/or select a record to delete !\n");
      return;
   }
   
   /* the selected row is already stored in the record_selected callback */
   on_recorddelete_clicked(NULL, createRecordstruct(p)); /* call callback */
}

void on_edit_modify_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   /* the selected row is stored in the record_selected callback */
   createModifyWindow((Params *)user_data);
}

void on_edit_copy_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   /* the selected row is stored in the record_selected callback */
   createCopyMainWindow((Params *)user_data);
}

void on_help_contents_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   ;
}

void on_help_about_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   createMessageBox("GEQUEL" GEQUELVERSION ": MySQL database\n"
		    "manipulation GUI written by Kees Lemmens\n"
		    "TU Delft, March 2003\n\n"
		    "Similar to the XForms based program\n "
		    "'xsqlmenu' by the same author in 1996\n\n"
		    "GNU public license (but use on\n"
		    "Micro$oft based systems is not advised ;-)");
}

/* Buttons in main window : */

void on_showfields_clicked(GtkButton *button, gpointer user_data)
{
   showFields((Params *)user_data);
}

void on_showrecords_clicked(GtkButton *button, gpointer user_data)
{
   showRecords((Params *)user_data);
}

void on_insert_clicked(GtkButton *button, gpointer user_data)
{
   on_edit_insert_activate(NULL, user_data); /* call other callback */
}

void on_delete_clicked(GtkButton *button, gpointer user_data)
{
   on_edit_delete_activate(NULL,user_data); /* call other callback */
}

void on_modify_clicked(GtkButton *button, gpointer user_data)
{
   on_edit_modify_activate(NULL,user_data); /* call other callback */
}

void on_copy_clicked(GtkButton *button, gpointer user_data)
{
   on_edit_copy_activate(NULL,user_data); /* call other callback */
}

void on_search_clicked(GtkButton *button, gpointer user_data)
{
   createSearchWindow((Params *)user_data);
}

void on_createtable_clicked(GtkButton *button, gpointer user_data)
{
   showMessage("Sorry: not implemented yet");
}

/* Text entries in main window : */

void on_query_activate(GtkEditable *editable, gpointer user_data)
{
   showQuery((Params *)user_data,
	     gtk_editable_get_chars(editable,0,-1));
}

void on_server_activate(GtkEditable *editable, gpointer user_data)
{
   setNewServer((Params *)user_data,
		gtk_entry_get_text(GTK_ENTRY(editable)));
}

void usage(char *name)
{
   fprintf(stderr,"\nUsage  : %s [options] [<database> [<table>]]\n",name);
   fprintf(stderr,  "Options: -h <host>   : set remote database server\n");
   fprintf(stderr,  "         -u <user>   : change userid   (MySQL)\n");
   fprintf(stderr,  "         -k <passwd> : supply password (MySQL)\n");
   fprintf(stderr,  "         -c          : disable record count display\n");
   exit(1);
}

/* Attach the widgets we need in the code to the Params structure */
void storeWidgets(GtkWidget *main, Params *p)
{
   p->querywindow  = GTK_TEXT  (lookup_widget (main, "querywindow"));
   p->logwindow    = GTK_TEXT  (lookup_widget (main, "logwindow"));
   
   p->notebook     = GTK_WIDGET(lookup_widget (main, "notebook"));

   p->clists[RECORDSTAB] = GTK_WIDGET(lookup_widget (main, "recordsclist"));
   p->clists[FIELDSTAB]  = GTK_WIDGET(lookup_widget (main, "fieldsclist"));
   p->clists[SEARCHTAB]  = GTK_WIDGET(lookup_widget (main, "searchclist"));
   p->clists[QUERYTAB]   = GTK_WIDGET(lookup_widget (main, "queryclist"));
   
   p->tabs[RECORDSTAB]   = GTK_WIDGET(lookup_widget (main, "recordstab"));
   p->tabs[FIELDSTAB]    = GTK_WIDGET(lookup_widget (main, "fieldstab"));
   p->tabs[SEARCHTAB]    = GTK_WIDGET(lookup_widget (main, "searchtab"));
   p->tabs[QUERYTAB]     = GTK_WIDGET(lookup_widget (main, "querytab"));
   
   p->dbaselist    = GTK_OPTION_MENU(lookup_widget (main, "dbaselist"));
   p->tablelist    = GTK_OPTION_MENU(lookup_widget (main, "tablelist"));
}

void connectMainWindowSignals(GtkWidget *main, Params *par)
{
   ldebug(2,"connectMainWindowSignals entered\n");
   ldebug(3,"Params pointer: %ld\n",(long)par);

   /* Menus : */
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_loadquery")), "activate",
		       GTK_SIGNAL_FUNC (on_file_loadquery_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_savequery")), "activate",
		       GTK_SIGNAL_FUNC (on_file_savequery_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_saverecords")), "activate",
		       GTK_SIGNAL_FUNC (on_file_saverecords_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_savelog")), "activate",
		       GTK_SIGNAL_FUNC (on_file_savelog_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_print")), "activate",
		       GTK_SIGNAL_FUNC (on_file_print_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_exit")), "activate",
		       GTK_SIGNAL_FUNC (on_file_exit_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edit_delete")), "activate",
		       GTK_SIGNAL_FUNC (on_edit_delete_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edit_modify")), "activate",
		       GTK_SIGNAL_FUNC (on_edit_modify_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edit_insert")), "activate",
		       GTK_SIGNAL_FUNC (on_edit_insert_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edit_copy")), "activate",
		       GTK_SIGNAL_FUNC (on_edit_copy_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"help_contents")), "activate",
		       GTK_SIGNAL_FUNC (on_help_contents_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"help_about")), "activate",
		       GTK_SIGNAL_FUNC (on_help_about_activate), par);
   
   /* Buttons : */
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"showfields")), "clicked",
		       GTK_SIGNAL_FUNC (on_showfields_clicked), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"showrecords")), "clicked",
		       GTK_SIGNAL_FUNC (on_showrecords_clicked), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"insert")), "clicked",
		       GTK_SIGNAL_FUNC (on_insert_clicked), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"delete")), "clicked",
		       GTK_SIGNAL_FUNC (on_delete_clicked), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"modify")), "clicked",
		       GTK_SIGNAL_FUNC (on_modify_clicked), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"copy")), "clicked",
		       GTK_SIGNAL_FUNC (on_copy_clicked), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"search")), "clicked",
		       GTK_SIGNAL_FUNC (on_search_clicked), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"createtable")), "clicked",
		       GTK_SIGNAL_FUNC (on_createtable_clicked), par);

   /* Textentries : */
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"querywindow")), "activate",
                      GTK_SIGNAL_FUNC (on_query_activate), par);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"server")), "activate",
                      GTK_SIGNAL_FUNC (on_server_activate), par);
}

void gequelInit(Params *p, int argc, char *argv[])
{
   int x;
   char *basename;
   double sortcolor[] =  { 0.8, 0.2, 0.1 }; /* red */
   
   /* WATCHOUT : GTK parses arguments first and may have removed those that are
    * recognized by GTKlib itself before we enter this routine !!!! */

   ldebug(2,"gequelInit called\n");
   basename=argv[0];
   p->countflag = COUNT;
   p->selectedrow  = -1;
   p->nrtabs = TABCOUNT;
   
   for(x=0;x<p->nrtabs;x++)
     p->tabinuse[x] = TRUE; /* they are the dummies created by glade */
   
   x=argc;
   while(--x>0)                    /* parse options */
   {
      if(*argv[1]=='-')
      {
	 switch(*(++argv[1]))
	 {
	  case 'c':  p->countflag = NOCOUNT;        break;
	  case 'h': if(argc <= 2) usage(basename);
	    p->server=argv[2]; ++argv; --argc; --x; break;
	  case 'u': if(argc <= 2) usage(basename);
	    p->user=argv[2];   ++argv; --argc; --x; break;
	  case 'k': if(argc <= 2) usage(basename);
	    p->pwd=argv[2];    ++argv; --argc; --x; break;
	  default:  usage(basename);      break;
	 }
	 ++argv; --argc;
      }
   }

   if(argc > 3)
     usage(basename);
   
   setNewServer(p,p->server);
   
   if(argc >= 2)
   {
      setDBaseList(p,argv[1]);
      setNewDatabase(p,argv[1]);
   }
   
   if(argc == 3)
   {
      setTableList(p,argv[2]);
      setNewTable(p, argv[2]);
   }

   p->styles[STLDEFAULT] = gtk_style_copy(gtk_widget_get_style(p->mainwindow));
   p->styles[STLRED]     = createColorStyle(p->styles[STLDEFAULT], sortcolor);
   p->styles[STLBOLD]    = createFontStyle(p->styles[STLDEFAULT], BOLDFONT);
   p->styles[STLBLDRED]  = createColorFontStyle(p->styles[STLDEFAULT],
						sortcolor, BOLDFONT);

   gtk_widget_set_style (lookup_widget(p->mainwindow,"titlelabel"),
			 p->styles[STLBOLD]);
   gtk_widget_set_style (lookup_widget(p->mainwindow,"serverlabel"),
			 p->styles[STLBOLD]);
   gtk_widget_set_style (lookup_widget(p->mainwindow,"dbaselabel"),
			 p->styles[STLBOLD]);
   gtk_widget_set_style (lookup_widget(p->mainwindow,"tablelabel"),
			 p->styles[STLBOLD]);
}
